SystemOrganization addCategory: #Jaiva!
SystemOrganization addCategory: #'Jaiva-Tests'!

Announcer subclass: #JaAnnouncer
	instanceVariableNames: 'stack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Jaiva'!

!JaAnnouncer methodsFor: 'announce' stamp: 'MartinDias 5/24/2013 19:46'!
announce: anAnnouncement after: aBlockClosure 

	| result theAnnouncement |
	theAnnouncement := anAnnouncement asAnnouncement.

	self stack push: theAnnouncement.
	result := [ 
		aBlockClosure ifCurtailed: [ 
			self announce: JaCurtailed ] ] 
		ensure: [ self stack pop ].

	self announce: theAnnouncement.
	^ result! !

!JaAnnouncer methodsFor: 'accessing' stamp: 'MartinDias 5/24/2013 19:46'!
stack
	^ stack ifNil: [ stack := Stack new ]! !

Announcement subclass: #JaAnotherAnnouncement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Jaiva-Tests'!

Announcement subclass: #JaCurtailed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Jaiva'!

Announcement subclass: #JaSomeAnnouncement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Jaiva-Tests'!

Job subclass: #JaJob
	instanceVariableNames: 'endAnnouncement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Jaiva'!

!JaJob methodsFor: 'private' stamp: 'MartinDias 5/27/2013 13:34'!
cleanupAfterRunning
	super cleanupAfterRunning.

	endAnnouncement isNil ifFalse: [
		SystemAnnouncer uniqueInstance announce: endAnnouncement ].! !

!JaJob methodsFor: 'accessing' stamp: 'MartinDias 5/27/2013 13:33'!
endAnnouncement
	^ endAnnouncement! !

!JaJob methodsFor: 'accessing' stamp: 'MartinDias 5/27/2013 13:33'!
endAnnouncement: anAnnouncement 
	endAnnouncement := anAnnouncement! !

!BlockClosure methodsFor: '*Jaiva' stamp: 'MartinDias 5/27/2013 13:32'!
asJaJob: anAnnouncement

	^ JaJob block: self.! !

TestCase subclass: #JaAnnouncerTest
	instanceVariableNames: 'announcer someAnnounces curtailedAnnounces anotherAnnounces'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Jaiva-Tests'!

!JaAnnouncerTest methodsFor: 'running' stamp: 'MartinDias 5/24/2013 19:07'!
setUp
	someAnnounces := OrderedCollection new.
	anotherAnnounces := OrderedCollection new.
	curtailedAnnounces := OrderedCollection new.
	announcer := JaAnnouncer new.
	
	announcer 
		on: JaSomeAnnouncement
		send: #add:
		to: someAnnounces.
	announcer 
		on: JaAnotherAnnouncement
		send: #add:
		to: anotherAnnounces.
	announcer 
		on: JaCurtailed
		send: #add:
		to: curtailedAnnounces.! !

!JaAnnouncerTest methodsFor: 'tests' stamp: 'MartinDias 5/24/2013 19:00'!
testSimpleCurtailed

	self should: [	
		announcer 
			announce: JaSomeAnnouncement
			after: [ 1/0 ] ]
		raise: ZeroDivide.
	
	self assert: curtailedAnnounces size equals: 1.
	self assert: someAnnounces isEmpty! !

!JaAnnouncerTest methodsFor: 'tests' stamp: 'MartinDias 5/24/2013 19:02'!
testSimpleSuccess

	| success |
	success := false.
	
	announcer 
		announce: JaSomeAnnouncement
		after: [ 
			self assert: someAnnounces isEmpty.
			success := true ].
	
	self assert: success.
	self assert: someAnnounces size equals: 1.
	self assert: curtailedAnnounces isEmpty! !

!JaAnnouncerTest methodsFor: 'tests' stamp: 'MartinDias 5/24/2013 19:29'!
testWhenCurtailedThePushedIsNotAnnounced

	self should: [	
		announcer 
			announce: JaSomeAnnouncement
			after: [ 
				3 timesRepeat: [ announcer announce: JaAnotherAnnouncement ].
				1/0 ] ]
		raise: ZeroDivide.

	self assert: someAnnounces isEmpty.
	self assert: anotherAnnounces size equals: 3.
	self assert: curtailedAnnounces size equals: 1.! !

!JaAnnouncerTest methodsFor: 'tests' stamp: 'MartinDias 5/24/2013 19:31'!
testWhenNestedAreAnnouncedTheStackHasPushedAnnouncement

	announcer
		on: JaAnotherAnnouncement
		do: [ :ann | 
			self assert: announcer stack size equals: 1.
			self assert: announcer stack top class equals: JaSomeAnnouncement ].

	announcer 
		announce: JaSomeAnnouncement
		after: [ announcer announce: JaAnotherAnnouncement ].
		
	self assert: announcer stack isEmpty! !

!JaAnnouncerTest methodsFor: 'tests' stamp: 'MartinDias 5/24/2013 19:28'!
testWhenSuccessThePushedIsAnnounced

	announcer 
		announce: JaSomeAnnouncement
		after: [ 
			3 timesRepeat: [ 
				announcer announce: JaAnotherAnnouncement ] ].
	
	self assert: someAnnounces size equals: 1.
	self assert: anotherAnnounces size equals: 3.
	self assert: curtailedAnnounces isEmpty.! !

!Compiler methodsFor: '*Jaiva' stamp: 'MartinDias 5/27/2013 13:35'!
evaluate: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: failBlock logged: logFlag
	"Compiles the sourceStream into a parse tree, then generates code into
	 a method. If aContext is not nil, the text can refer to temporaries in that
	 context (the Debugger uses this). If aRequestor is not nil, then it will receive
	 a notify:at: message before the attempt to evaluate is aborted. Finally, the 
	 compiled method is invoked from here via withArgs:executeMethod:, hence
	 the system no longer creates Doit method litter on errors."

	| methodNode method value toLog itsSelection itsSelectionString |
	class := aContext == nil ifTrue: [receiver class ] ifFalse: [aContext method methodClass].
	self from: textOrStream class: class context: aContext notifying: aRequestor.
	methodNode := self translate: sourceStream noPattern: true ifFail: [^failBlock value].

	method := self interactive
				ifTrue: [methodNode generateWithSource]
				ifFalse: [methodNode generate].

	([ :job | 
	value := receiver
				withArgs: (context ifNil: [#()] ifNotNil: [{context}])
				executeMethod: method.

	logFlag ifTrue:
		[toLog := ((requestor respondsTo: #selection)  
			and:[(itsSelection := requestor selection) notNil
			and:[(itsSelectionString := itsSelection asString) isEmptyOrNil not]])
				ifTrue:[itsSelectionString]
				ifFalse:[sourceStream contents].
		job endAnnouncement:
			(ExpressionEvaluated 
				expression: toLog
				context: aContext) ] ] asJaJob: nil) run.
	^ value! !
